# Various extensions and modifications to Webrick.

# A nice, XHTML 1.1 valid error page
module WEBrick
  class HTTPResponse
    def set_error(ex, backtrace=false)
      case ex
      when HTTPStatus::Status 
        @keep_alive = false if HTTPStatus::error?(ex.code)
        self.status = ex.code
      else 
        @keep_alive = false
        self.status = HTTPStatus::RC_INTERNAL_SERVER_ERROR
      end
      @header['content-type'] = "text/html"

      if respond_to?(:create_error_page)
        create_error_page()
        return
      end

      if @request_uri
        host, port = @request_uri.host, @request_uri.port
      else
        host, port = @config[:ServerName], @config[:Port]
      end

      @body = ''
      @body << <<-_end_of_html_
<?xml version="1.0" encoding="iso-8859-1"?>
<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.1//EN"
                      "http://www.w3.org/TR/xhtml11/DTD/xhtml11.dtd">
<html xmlns="http://www.w3.org/1999/xhtml" xml:lang="en">
  <head>
    <title>#{HTMLUtils::escape(@reason_phrase)}</title>
    <style>
body { font-family: "Trebuchet MS", Times, serif; }
hr { height: 1px; color: black; clear: both; }
div.message { white-space: pre; }
#num { float: left; font-size: 2.5em; margin-right: 0.2em; }
    </style>
  </head>
  <body>
    <h1><span id="num">#{self.status}</span> #{HTMLUtils::escape(@reason_phrase)}</h1>
    <div class="message">#{HTMLUtils::escape(ex.message)}</div>
    <hr />
      _end_of_html_

      if backtrace && $DEBUG
        @body << "backtrace of `#{HTMLUtils::escape(ex.class.to_s)}' "
        @body << "#{HTMLUtils::escape(ex.message)}"
        @body << "<pre>"
        ex.backtrace.each{|line| @body << "\t#{line}\n"}
        @body << "</pre>\n<hr />"
      end

      @body << <<-_end_of_html_
    <address>
     #{HTMLUtils::escape(@config[:ServerSoftware])} at
     #{host}:#{port}
    </address>
  </body>
</html>
      _end_of_html_
    end
  end
end
